@@ -6,7 +6,6 @@ This file contains messages outputed to the screen.
 
 from ..version import version as __version__
 from ..version import date as __date__
-from prettytable import PrettyTable
 
 
 def print_mechelastic():
@@ -50,14 +49,7 @@ def print_warning_2D():
     return
 
 
-# def printMatrix(c):
-#     p = PrettyTable()
-#     for row in c:
-#         p.add_row(row)
-#     print(p.get_string(header=False, border=False))
-
-
-def printMatrix(c):
+def print_matrix(c):
     row = c.shape[0]
     col = c.shape[1]
     for i in range(row):
