@@ -426,11 +426,11 @@ def maximize(func, dim):
 
 
 class ELATE:
-    def __init__(self, s):
+    def __init__(self, s,density = None):
 
         self.elas = Elastic(s)
         self.elasList = s
-
+        self.density = density
         minE = minimize(self.elas.Young, 2)
         maxE = maximize(self.elas.Young, 2)
         minLC = minimize(self.elas.LC, 2)
@@ -614,6 +614,128 @@ class ELATE:
         )
 
         return data
+    
+    def COMPRESSION_SPEED2D(self, npoints):
+        data1 = makePolarPlot2(
+            lambda x: self.elas.compressionSpeed2D([np.pi / 2, x],density = self.density),
+            "Compression Speed in (xy) plane",
+            "xy",
+            npoints=npoints,
+        )
+        data2 = makePolarPlot2(
+            lambda x: self.elas.compressionSpeed2D([x, 0],density = self.density), "Compression Speed in (xz) plane", "xz"
+        )
+        data3 = makePolarPlot2(
+            lambda x: self.elas.compressionSpeed2D([x, np.pi / 2], density = self.density),
+            "Compression Speed in (yz) plane",
+            "yz",
+            npoints=npoints,
+        )
+        return (data1, data2, data3)
+    
+    def COMPRESSION_SPEED3D(self, npoints):
+
+        if self.elas.isOrthorhombic():
+            self.elas = ElasticOrtho(self.elas)
+
+        data = make3DPlot2(
+            lambda x, y, g1, g2: self.elas.compressionSpeed3D(x, y, g1, g2, density = self.density),
+            "Compression speed",
+            npoints=npoints,
+        )
+
+        return data
+    
+    def SHEAR_SPEED2D(self, npoints):
+        data1 = makePolarPlot2(
+            lambda x: self.elas.shearSpeed2D([np.pi / 2, x],density = self.density),
+            "Shear speed in (xy) plane",
+            "xy",
+            npoints=npoints,
+        )
+        data2 = makePolarPlot2(
+            lambda x: self.elas.shearSpeed2D([x, 0],density = self.density), "Shear speed in (xz) plane", "xz"
+        )
+        data3 = makePolarPlot2(
+            lambda x: self.elas.shearSpeed2D([x, np.pi / 2],density = self.density),
+            "Shear speed in (yz) plane",
+            "yz",
+            npoints=npoints,
+        )
+        return (data1, data2, data3)
+    
+    def SHEAR_SPEED3D(self, npoints):
+
+        if self.elas.isOrthorhombic():
+            self.elas = ElasticOrtho(self.elas)
+
+        data = make3DPlot2(
+            lambda x, y, g1, g2: self.elas.shearSpeed3D(x, y, g1, g2, density = self.density),
+            "Shear speed",
+            npoints=npoints,
+        )
+        return data
+    
+    def RATIO_COMPRESSIONAL_SHEAR2D(self, npoints):
+        data1 = makePolarPlot2(
+            lambda x: self.elas. ratio_compressional_shear2D([np.pi / 2, x],density = self.density),
+            "Ratio compressional/shear in (xy) plane",
+            "xy",
+            npoints=npoints,
+        )
+        data2 = makePolarPlot2(
+            lambda x: self.elas. ratio_compressional_shear2D([x, 0],density = self.density), "Ratio compressional/shear in (xz) plane", "xz"
+        )
+        data3 = makePolarPlot2(
+            lambda x: self.elas. ratio_compressional_shear2D([x, np.pi / 2],density = self.density),
+            "Ratio compressional/shear in (yz) plane",
+            "yz",
+            npoints=npoints,
+        )
+        return (data1, data2, data3)
+        
+    def RATIO_COMPRESSIONAL_SHEAR3D(self, npoints):
+
+        if self.elas.isOrthorhombic():
+            self.elas = ElasticOrtho(self.elas)
+
+        data = make3DPlot2(
+            lambda x, y, g1, g2: self.elas.ratio_compressional_shear3D(x, y, g1, g2,density = self.density),
+            "Ratio of compression/shear",
+            npoints=npoints,
+        )
+        return data
+    
+    def DEBYE_SPEED2D(self, npoints):
+        data1 = makePolarPlot2(
+            lambda x: self.elas.debyeSpeed2D([np.pi / 2, x],density = self.density),
+            "Debye speed in (xy) plane",
+            "xy",
+            npoints=npoints,
+        )
+        data2 = makePolarPlot2(
+            lambda x: self.elas.debyeSpeed2D([x, 0], density = self.density), "Debye speed in (xz) plane", "xz"
+        )
+        data3 = makePolarPlot2(
+            lambda x: self.elas.debyeSpeed2D([x, np.pi / 2],density = self.density),
+            "Debye speed in (yz) plane",
+            "yz",
+            npoints=npoints,
+        )
+        return (data1, data2, data3)
+    
+    def DEBYE_SPEED3D(self, npoints):
+
+        if self.elas.isOrthorhombic():
+            self.elas = ElasticOrtho(self.elas)
+
+        data = make3DPlot2(
+            lambda x, y, g1, g2: self.elas.debyeSpeed3D(x, y, g1, g2,density = self.density),
+            "Debye Speed",
+            npoints=npoints,
+        )
+
+        return data
 
     ##############################################################################
     # Plotting functions
@@ -702,6 +824,82 @@ class ELATE:
                         plotter.add_mesh(grid, opacity=0.25, color=icolor)
                     else:
                         plotter.add_mesh(grid, opacity=0.50, color=icolor)
+                        
+        if self.density != None:    
+            if elastic_calc == "COMPRESSION_SPEED":
+                func = self.COMPRESSION_SPEED3D(npoints=100)
+                colors = ["green", "blue"]
+                for ix, icolor in zip(range(len(func)), colors):
+                    x = np.array(func[ix][0])
+                    y = np.array(func[ix][1])
+                    z = np.array(func[ix][2])
+                    r = np.array(func[ix][3])
+                    if np.all((func[ix][0] == 0)):
+                        continue
+                    else:
+                        grid = pv.StructuredGrid(x, y, z)
+                        meshes.append(grid)
+                        if ix == 2:
+                            plotter.add_mesh(grid, opacity=0.25, color=icolor)
+                        else:
+                            plotter.add_mesh(grid, opacity=0.50, color=icolor)
+            
+            elif elastic_calc == "SHEAR_SPEED":
+                func = self.SHEAR_SPEED3D(npoints=100)
+                colors = ["green", "blue"]
+                for ix, icolor in zip(range(len(func)), colors):
+                    x = np.array(func[ix][0])
+                    y = np.array(func[ix][1])
+                    z = np.array(func[ix][2])
+                    r = np.array(func[ix][3])
+                    if np.all((func[ix][0] == 0)):
+                        continue
+                    else:
+                        grid = pv.StructuredGrid(x, y, z)
+                        meshes.append(grid)
+                        if ix == 2:
+                            plotter.add_mesh(grid, opacity=0.25, color=icolor)
+                        else:
+                            plotter.add_mesh(grid, opacity=0.50, color=icolor)
+                            
+            elif elastic_calc == "RATIO_COMPRESSIONAL_SHEAR":
+                func = self.RATIO_COMPRESSIONAL_SHEAR3D(npoints=100)
+                colors = ["green", "blue"]
+                for ix, icolor in zip(range(len(func)), colors):
+                    x = np.array(func[ix][0])
+                    y = np.array(func[ix][1])
+                    z = np.array(func[ix][2])
+                    r = np.array(func[ix][3])
+                    if np.all((func[ix][0] == 0)):
+                        continue
+                    else:
+                        grid = pv.StructuredGrid(x, y, z)
+                        meshes.append(grid)
+                        if ix == 2:
+                            plotter.add_mesh(grid, opacity=0.25, color=icolor)
+                        else:
+                            plotter.add_mesh(grid, opacity=0.50, color=icolor)
+                            
+            elif elastic_calc == "DEBYE_SPEED":
+                func = self.DEBYE_SPEED3D(npoints=100)
+                colors = ["green", "blue"]
+                for ix, icolor in zip(range(len(func)), colors):
+                    x = np.array(func[ix][0])
+                    y = np.array(func[ix][1])
+                    z = np.array(func[ix][2])
+                    r = np.array(func[ix][3])
+                    if np.all((func[ix][0] == 0)):
+                        continue
+                    else:
+                        grid = pv.StructuredGrid(x, y, z)
+                        meshes.append(grid)
+                        if ix == 2:
+                            plotter.add_mesh(grid, opacity=0.25, color=icolor)
+                        else:
+                            plotter.add_mesh(grid, opacity=0.50, color=icolor)
+                            
+        if elastic_calc in ['DEBYE_SPEED', 'SHEAR_SPEED', 'COMPRESSIONAL_SPEED', 'RATIO_COMPRESSIONAL_SHEAR'] and self.density == None:
+            print("You must specify density in kg/m^3 to produce DEBYE_SPEED, SHEAR_SPEED, COMPRESSIONAL_SPEED, and RATIO_COMPRESSIONAL_SHEAR")
 
         plotter.add_axes()
         if show:
@@ -709,7 +907,7 @@ class ELATE:
         return meshes
         #plotter.show_grid(color = "black")
 
-    def plot_2D(self, elastic_calc="all", npoints=100, apply_to_plot=None, show= True):
+    def plot_2D(self, elastic_calc="all", npoints=100, apply_to_plot=None, density= None, show= True, ):
         """
 
 
@@ -904,6 +1102,72 @@ class ELATE:
                     )
                 if apply_to_plot is not None:
                     apply_to_plot(fig, ax)
+                    
+        if density != None :
+            
+            if elastic_calc == "COMPRESSION_SPEED":
+                func = self.COMPRESSION_SPEED2D(npoints=npoints)
+                colors = ["green", "blue"]
+                labels = ["Compression speed - Max", "Compression speed -"]
+                fig.suptitle("Compression speed")
+              
+                for iplane, title in zip(range(len(func)), subTitles):
+    
+                    ax = fig.add_subplot(1, 3, iplane + 1)
+                    ax.set_title(title)
+                    ax.get_yaxis().set_visible(False)
+                    for iplot, color in zip(range(len(func[0])), colors):
+                        plt.plot(
+                            func[iplane][iplot][0], func[iplane][iplot][1], color=color
+                        )
+            if elastic_calc == "SHEAR_SPEED":
+                func = self.SHEAR_SPEED2D(npoints=npoints)
+                colors = ["green", "blue"]
+                labels = ["Shear speed - Max", "Shear speed -"]
+                fig.suptitle("Shear speed")
+                for iplane, title in zip(range(len(func)), subTitles):
+    
+                    ax = fig.add_subplot(1, 3, iplane + 1)
+                    ax.set_title(title)
+                    ax.get_yaxis().set_visible(False)
+                    for iplot, color in zip(range(len(func[0])), colors):
+                        plt.plot(
+                            func[iplane][iplot][0], func[iplane][iplot][1], color=color
+                        )
+            if elastic_calc == "RATIO_COMPRESSIONAL_SHEAR":
+                func = self.RATIO_COMPRESSIONAL_SHEAR2D(npoints=npoints)
+                colors = ["green", "blue"]
+                labels = ["Ratio compression/shear - Max", "Ratio compression/shear -"]
+                fig.suptitle("Ratio compression/shear")
+                for iplane, title in zip(range(len(func)), subTitles):
+    
+                    ax = fig.add_subplot(1, 3, iplane + 1)
+                    ax.set_title(title)
+                    ax.get_yaxis().set_visible(False)
+                    for iplot, color in zip(range(len(func[0])), colors):
+                        plt.plot(
+                            func[iplane][iplot][0], func[iplane][iplot][1], color=color
+                        )
+            if elastic_calc == "DEBYE_SPEED":
+                func = self.DEBYE_SPEED2D(npoints=npoints)
+                colors = ["green", "blue"]
+                labels = ["Debye speed - Max", "Debye speed -"]
+                fig.suptitle("Debye speed")
+                for iplane, title in zip(range(len(func)), subTitles):
+    
+                    ax = fig.add_subplot(1, 3, iplane + 1)
+                    ax.set_title(title)
+                    ax.get_yaxis().set_visible(False)
+                    for iplot, color in zip(range(len(func[0])), colors):
+                        plt.plot(
+                            func[iplane][iplot][0], func[iplane][iplot][1], color=color
+                        )
+        if elastic_calc in ['DEBYE_SPEED', 'SHEAR_SPEED', 'COMPRESSIONAL_SPEED', 'RATIO_COMPRESSIONAL_SHEAR'] and density == None:
+            print("You must specify density in kg/m^3 to produce DEBYE_SPEED, SHEAR_SPEED, COMPRESSIONAL_SPEED, and RATIO_COMPRESSIONAL_SHEAR")
+
+            
+            
+            
         if show:
             plt.show()
             
@@ -1086,7 +1350,7 @@ class ELATE:
 class Elastic:
     """An elastic tensor, along with methods to access it"""
 
-    def __init__(self, s):
+    def __init__(self, s ):
         """Initialize the elastic tensor from a string"""
 
         if not s:
@@ -1441,6 +1705,220 @@ class Elastic:
             float(r1.x),
             float(r2.x),
         )
+    
+    
+    def compressionSpeed2D(self, x, density = None):
+        ftol = 0.001
+        xtol = 0.01
+
+        def func1(z):
+            return (3*self.averages()[2][0]*(10**9)*(1-self.Poisson([x[0], x[1], z]))/(2*density*(1+self.Poisson([x[0], x[1], z]))))**0.5
+
+
+        r1 = optimize.minimize(
+            func1,
+            np.pi / 2.0,
+            args=(),
+            method="Powell",
+            options={"xtol": xtol, "ftol": ftol},
+        )  # , bounds=[(0.0,np.pi)])
+
+        def func2(z):
+            return -(3*self.averages()[2][0]*(10**9)*(1-self.Poisson([x[0], x[1], z]))/(2*density*(1+self.Poisson([x[0], x[1], z]))))**0.5
+
+
+        r2 = optimize.minimize(
+            func2,
+            np.pi / 2.0,
+            args=(),
+            method="Powell",
+            options={"xtol": xtol, "ftol": ftol},
+        )  # , bounds=[(0.0,np.pi)])
+        return (float(r1.fun), -float(r2.fun))
+
+    
+    def compressionSpeed3D(self, x, y, guess1=np.pi / 2.0, guess2=np.pi / 2.0, density = None):
+        tol = 0.005
+
+        def func1(z):
+            return (3*self.averages()[2][0]*(10**9)*(1-self.Poisson([x, y, z]))/(2*density*(1+self.Poisson([x, y, z]))))**0.5
+
+        r1 = optimize.minimize(
+            func1, guess1, args=(), method="COBYLA", options={"tol": tol}
+        )  # , bounds=[(0.0,np.pi)])
+
+        def func2(z):
+            return -(3*self.averages()[2][0]*(10**9)*(1-self.Poisson([x, y, z]))/(2*density*(1+self.Poisson([x, y, z]))))**0.5
+
+        r2 = optimize.minimize(
+            func2, guess2, args=(), method="COBYLA", options={"tol": tol}
+        )  # , bounds=[(0.0,np.pi)])
+        return (float(r1.fun), -float(r2.fun), float(r1.x), float(r2.x))
+    
+    def shearSpeed2D(self, x, density = None):
+        ftol = 0.001
+        xtol = 0.01
+
+        def func1(z):
+            return ((1/density)*(10**9)*self.shear([x[0], x[1], z]))**0.5
+        r1 = optimize.minimize(
+            func1,
+            np.pi / 2.0,
+            args=(),
+            method="Powell",
+            options={"xtol": xtol, "ftol": ftol},
+        )  # , bounds=[(0.0,np.pi)])
+
+        def func2(z):
+            return -((1/density)*(10**9)*self.shear([x[0], x[1], z]))**0.5
+
+        r2 = optimize.minimize(
+            func2,
+            np.pi / 2.0,
+            args=(),
+            method="Powell",
+            options={"xtol": xtol, "ftol": ftol},
+        )  # , bounds=[(0.0,np.pi)])
+        return (float(r1.fun), -float(r2.fun))
+
+    
+    def shearSpeed3D(self, x, y, guess1=np.pi / 2.0, guess2=np.pi / 2.0, density=None):
+        tol = 0.005
+
+        def func1(z):
+            return ((1/density)*(10**9)*self.shear([x, y, z]))**0.5
+
+        r1 = optimize.minimize(
+            func1, guess1, args=(), method="COBYLA", options={"tol": tol}
+        )  # , bounds=[(0.0,np.pi)])
+
+        def func2(z):
+            return ((1/density)*(10**9)*self.shear([x, y, z]))**0.5
+
+        r2 = optimize.minimize(
+            func2, guess2, args=(), method="COBYLA", options={"tol": tol}
+        )  # , bounds=[(0.0,np.pi)])
+        return (float(r1.fun), -float(r2.fun), float(r1.x), float(r2.x))
+    
+    
+    def ratio_compressional_shear2D(self, x, density = None):
+        ftol = 0.001
+        xtol = 0.01
+
+        def v_p(z):
+            return (3*self.averages()[2][0]*(10**9)*(1-self.Poisson([x[0], x[1], z]))/(2*density*(1+self.Poisson([x[0], x[1], z]))))**0.5
+
+        def v_s(z):
+            return ((1/density)*(10**9)*self.shear([x[0], x[1], z]))**0.5
+        
+        
+        def func1(z):
+            return (v_p(z) /v_s(z))**2
+
+        r1 = optimize.minimize(
+            func1,
+            np.pi / 2.0,
+            args=(),
+            method="Powell",
+            options={"xtol": xtol, "ftol": ftol},
+        )  # , bounds=[(0.0,np.pi)])
+
+        def func2(z):
+            return -(v_p(z) /v_s(z))**2
+
+        r2 = optimize.minimize(
+            func2,
+            np.pi / 2.0,
+            args=(),
+            method="Powell",
+            options={"xtol": xtol, "ftol": ftol},
+        )  # , bounds=[(0.0,np.pi)])
+        return (float(r1.fun), -float(r2.fun))
+
+    
+    def ratio_compressional_shear3D(self, x, y, guess1=np.pi / 2.0, guess2=np.pi / 2.0,density = None):
+        tol = 0.005
+
+
+        def v_p(z):
+            return (3*self.averages()[2][0]*(10**9)*(1-self.Poisson([x, y, z]))/(2*density*(1+self.Poisson([x, y, z]))))**0.5
+
+        def v_s(z):
+            return ((1/density)*(10**9)*self.shear([x, y, z]))**0.5
+        
+        
+        def func1(z):
+            return (v_p(z) /v_s(z))**2
+
+        r1 = optimize.minimize(
+            func1, guess1, args=(), method="COBYLA", options={"tol": tol}
+        )  # , bounds=[(0.0,np.pi)])
+
+        def func2(z):
+            return -(v_p(z) /v_s(z))**2
+
+        r2 = optimize.minimize(
+            func2, guess2, args=(), method="COBYLA", options={"tol": tol}
+        )  # , bounds=[(0.0,np.pi)])
+        return (float(r1.fun), -float(r2.fun), float(r1.x), float(r2.x))
+    
+    
+    def debyeSpeed2D(self, x,density = None):
+        ftol = 0.001
+        xtol = 0.01
+
+        def v_p(z):
+            return (3*self.averages()[2][0]*(10**9)*(1-self.Poisson([x[0], x[1], z]))/(2*density*(1+self.Poisson([x[0], x[1], z]))))**0.5
+
+        def v_s(z):
+            return ((1/density)*(10**9)*self.shear([x[0], x[1], z]))**0.5
+        
+        def func1(z):
+            return v_p(z)*v_s(z)/(2*v_s(z)**3 +v_p(z)**3)**(1/3)
+        r1 = optimize.minimize(
+            func1,
+            np.pi / 2.0,
+            args=(),
+            method="Powell",
+            options={"xtol": xtol, "ftol": ftol},
+        )  # , bounds=[(0.0,np.pi)])
+
+        def func2(z):
+            return -v_p(z)*v_s(z)/(2*v_s(z)**3 +v_p(z)**3)**(1/3)
+
+        r2 = optimize.minimize(
+            func2,
+            np.pi / 2.0,
+            args=(),
+            method="Powell",
+            options={"xtol": xtol, "ftol": ftol},
+        )  # , bounds=[(0.0,np.pi)])
+        return (float(r1.fun), -float(r2.fun))
+
+    
+    def debyeSpeed3D(self, x, y, guess1=np.pi / 2.0, guess2=np.pi / 2.0,density= None):
+        tol = 0.005
+        
+        def v_p(z):
+            return (3*self.averages()[2][0]*(10**9)*(1-self.Poisson([x, y, z]))/(2*density*(1+self.Poisson([x, y, z]))))**0.5
+
+        def v_s(z):
+            return ((1/density)*(10**9)*self.shear([x, y, z]))**0.5
+        
+        def func1(z):
+            return v_p(z)*v_s(z)/(2*v_s(z)**3 +v_p(z)**3)**(1/3)
+
+        r1 = optimize.minimize(
+            func1, guess1, args=(), method="COBYLA", options={"tol": tol}
+        )  # , bounds=[(0.0,np.pi)])
+
+        def func2(z):
+            return -v_p(z)*v_s(z)/(2*v_s(z)**3 +v_p(z)**3)**(1/3)
+
+        r2 = optimize.minimize(
+            func2, guess2, args=(), method="COBYLA", options={"tol": tol}
+        )  # , bounds=[(0.0,np.pi)])
+        return (float(r1.fun), -float(r2.fun), float(r1.x), float(r2.x))
 
 
 class ElasticOrtho(Elastic):
