@@ -8,10 +8,13 @@ import sys
 from intersect import intersection
 import networkx as nx
 import matplotlib as mpl
+import json
+from dicttoxml import dicttoxml
+from xml.dom.minidom import parseString
 
 # Setting up plotting class
 plt.rcParams["mathtext.default"] = "regular"
-plt.rcParams["font.family"] = "Georgia"
+plt.rcParams["font.family"] = "Arial"
 plt.rc("font", size=22)  # controls default text sizes
 plt.rc("axes", titlesize=22)  # fontsize of the axes title
 plt.rc("axes", labelsize=22)  # fontsize of the x and y labels
@@ -152,6 +155,7 @@ class EOS:
         vlim=None,
         model=None,
         raw_data=True,
+        export=False,
     ):
         """plot_eos.
 
@@ -186,6 +190,9 @@ class EOS:
             raw_data : bool, optional (default ``True``)
                 Also include the raw input data in the plot.
 
+            export : bool, optional (default ``False``)
+                Export data as .json and .xml.
+
         Returns
         -------
             param : None
@@ -197,6 +204,7 @@ class EOS:
         self.vlim = vlim
         self.model = model
         self.raw_data = raw_data
+        self.export = export
 
         if self.eostype == "energy":
 
@@ -398,11 +406,11 @@ class EOS:
                 )
 
             if self.au:
-                axs.set_xlabel("Volume ($Bohr^3$)/atom")
-                axs.set_ylabel("Energy (Ha)/atom")
+                axs.set_xlabel("Volume ($Bohr^3$/atom)")
+                axs.set_ylabel("Energy (Ha/atom)")
             else:
-                axs.set_xlabel("Volume ($\AA^3$)/atom")
-                axs.set_ylabel("Energy (eV)/atom")
+                axs.set_xlabel("Volume ($\AA^3$/atom)")
+                axs.set_ylabel("Energy (eV/atom)")
             axs.set_title("Energy vs. Volume")
             plt.legend(loc="best")
             plt.show()
@@ -485,9 +493,9 @@ class EOS:
                 )
 
             if self.au:
-                axs2.set_xlabel("Volume ($Bohr^3$)/atom")
+                axs2.set_xlabel("Volume ($Bohr^3$/atom)")
             else:
-                axs2.set_xlabel("Volume ($\AA^3$)/atom")
+                axs2.set_xlabel("Volume ($\AA^3$/atom)")
             axs2.set_ylabel("Pressure (GPa)")
             axs2.set_title("Pressure vs. Volume")
             plt.legend(loc="best")
@@ -541,9 +549,9 @@ class EOS:
                 )
 
             if self.au:
-                axs.set_xlabel("Volume ($Bohr^3$)/atom")
+                axs.set_xlabel("Volume ($Bohr^3$/atom)")
             else:
-                axs.set_xlabel("Volume ($\AA^3$)/atom")
+                axs.set_xlabel("Volume ($\AA^3$/atom)")
             axs.set_ylabel("Pressure (GPa)")
             axs.set_title("Pressure vs. Volume")
             plt.legend(loc="best")
@@ -574,15 +582,20 @@ class EOS:
                 )
 
             if self.au:
-                axs2.set_ylabel("Energy (Ha)/atom")
-                axs2.set_xlabel("Volume ($Bohr^3$)/atom")
+                axs2.set_ylabel("Energy (Ha/atom)")
+                axs2.set_xlabel("Volume ($Bohr^3$/atom)")
             else:
-                axs2.set_ylabel("Energy (eV)/atom")
-                axs2.set_xlabel("Volume ($\AA^3$)/atom")
+                axs2.set_ylabel("Energy (eV/atom)")
+                axs2.set_xlabel("Volume ($\AA^3$/atom)")
             axs2.set_title("Energy vs. Volume")
             plt.legend(loc="best")
             plt.show()
 
+        # Saving as .json and .xml
+        if self.export:
+            self.to_json()
+            self.to_xml()
+
     def plot_enthalpy_curves(
         self,
         infiles=None,
@@ -591,6 +604,8 @@ class EOS:
         vlim=None,
         vlim_list=None,
         deltaH_index=None,
+        labels=None,
+        export=False,
     ):
         """plot_enthalpy_curves.
 
@@ -610,6 +625,10 @@ class EOS:
             natoms : int, optional (default ``1``)
                 Number of atoms.
 
+            labels : str, optional (default ``None``)
+                List of names of phases. Supports Latex.
+                If not provided infiles will be used.
+
             au : bool, optional (default ``False``)
                 Output in atomic units?
 
@@ -622,6 +641,9 @@ class EOS:
             deltaH_index : int, optional (default ``None``)
                 Index of file to calculate enthalpy difference with respect to.
 
+            export : bool, optional (default ``False``)
+                Export data as .json and .xml.
+
         Returns
         -------
             param : None
@@ -634,6 +656,8 @@ class EOS:
         self.vlim = vlim
         self.vlim_list = vlim_list
         self.deltaH_index = deltaH_index
+        self.labels = labels
+        self.export = export
 
         nfiles = len(self.infiles)
         self.volume = []
@@ -970,57 +994,69 @@ class EOS:
         #             ),
         #         )
 
+        if self.labels is None:
+            self.labels = self.infiles
+
         for i, filename in enumerate(self.infiles):
-            axs.plot(self.pressure[i], self.H[i], label=filename)
+            axs.plot(self.pressure[i], self.H[i], label=self.labels[i])
         axs.set_xlabel("Pressure (GPa)")
         if self.au:
-            axs.set_ylabel("Enthalpy (Ha)/atom")
+            axs.set_ylabel("Enthalpy (Ha/atom)")
         else:
-            axs.set_ylabel("Enthalpy (eV)/atom")
+            axs.set_ylabel("Enthalpy (eV/atom)")
         axs.set_title("Enthalpy vs. Pressure")
         plt.legend(loc="best")
+        axs.grid(color="gainsboro", ls="--", lw=0.6)
+        plt.savefig("enthalpy.pdf")
         plt.show()
 
-        # Network map for phase transitions from Pedram
-        cmap = plt.get_cmap("winter")
+        # Network map for phase transitions
+        # Added by Pedram
+        cmap = plt.get_cmap("nipy_spectral")
         DG = nx.DiGraph()
         names = np.unique(self.trans_mat_reordered[:, 0:2])
+
         DG.add_nodes_from(names)
         for trans in self.trans_mat_reordered:
             DG.add_edge(
                 trans[0],
                 trans[1],
-                weight=trans[2],
+                weight=trans[3],
             )
         pos = nx.layout.spring_layout(DG)
 
-        node_sizes = [
-            500
-        ] * DG.number_of_nodes()  # [3 + 10 * i for i in range(len(DG))]
-        M = DG.number_of_edges()
-        edge_colors = self.trans_mat_reordered[:, 3]
+        node_sizes = [3500] * DG.number_of_nodes()
+        edge_colors = [
+            DG.get_edge_data(edge[0], edge[1])["weight"] for edge in DG.edges
+        ]  # self.tras_mat_reordered[:, 3]
 
-        # edge_alphas = transition_matrix[:,2]
-        plt.figure(figsize=(9, 6))
-        nodes = nx.draw_networkx_nodes(DG, pos, node_size=node_sizes, node_color="Gray")
+        plt.figure(figsize=(13, 9))
+        nodes = nx.draw_networkx_nodes(
+            DG, pos, node_size=node_sizes, node_color="gainsboro"
+        )
         edges = nx.draw_networkx_edges(
             DG,
             pos,
             node_size=node_sizes,
-            arrowstyle="->",
+            # arrowstyle="->",
             arrowsize=16,
             edge_color=edge_colors,
             edge_cmap=cmap,
-            width=2,
+            width=4,
         )
 
         nx.draw_networkx_labels(DG, pos, font_size=16)
 
         pc = mpl.collections.PatchCollection(edges, cmap=cmap)
         pc.set_array(edge_colors)
-        plt.colorbar(pc)
+        cb = plt.colorbar(pc, orientation="vertical")
+        if self.au:
+            cb.set_label("Enthalpy (Ha/atom)")
+        else:
+            cb.set_label("Enthalpy (eV/atom)")
         ax = plt.gca()
         ax.set_axis_off()
+        plt.savefig("enthalpy-network.pdf")
         plt.show()
 
         # Enthalpy differences with respect to a selected phase
@@ -1056,12 +1092,12 @@ class EOS:
             axs2 = fig.add_subplot(111)
 
             for i, filename in enumerate(self.infiles):
-                axs2.plot(self.pressure[i], self.deltaH[i], label=filename)
+                axs2.plot(self.pressure[i], self.deltaH[i], label=self.labels[i])
             axs2.set_xlabel("Pressure (GPa)")
             if self.au:
-                axs2.set_ylabel("$\Delta$H (Ha)/atom")
+                axs2.set_ylabel("$\Delta$H (Ha/atom)")
             else:
-                axs2.set_ylabel("$\Delta$H (eV)/atom")
+                axs2.set_ylabel("$\Delta$H (eV/atom)")
             title_string = "$\Delta$H vs. Pressure wrt " + str(phase_name)
             axs2.set_title(title_string)
             # axs2.axhline(color="black")
@@ -1069,6 +1105,11 @@ class EOS:
             plt.legend(loc="best")
             plt.show()
 
+        # Saving as .json and .xml
+        if self.export:
+            self.to_json()
+            self.to_xml()
+
     ############################# FITTING #############################
 
     def fitting(self):
@@ -1261,7 +1302,7 @@ class EOS:
 
     ####################### EOS MODELS for Energy ##############################
 
-    # Murnaghan equation of state
+    # Murnaghan
     def eos_murnaghan(self, coeffs, vol):
         "Ref: Phys. Rev. B 28, 5480 (1983)"
         E0, B0, Bp, V0 = coeffs
@@ -1272,7 +1313,7 @@ class EOS:
         )
         return E
 
-    # Birch-Murnaghan equation of state
+    # Birch-Murnaghan
     def eos_birch_murnaghan(self, coeffs, vol):
         "Ref: Phys. Rev. B 70, 224107"
         E0, B0, Bp, V0 = coeffs
@@ -1282,7 +1323,7 @@ class EOS:
         )
         return E
 
-    # Birch equation of state
+    # Birch
     def eos_birch(self, coeffs, vol):
         """
         Ref: Michael J. Mehl; Barry M. Klein; Dimitris A. Papaconstantopoulos. First-Principles Calculation of Elastic Properties. In Intermetallic Compounds; John Wiley & Sons Ltd, 1994; Vol. 1.
@@ -1296,7 +1337,7 @@ class EOS:
         )
         return E
 
-    # Vinet equation of state
+    # Vinet
     def eos_vinet(self, coeffs, vol):
         "Ref: Phys. Rev. B 70, 224107"
         E0, B0, Bp, V0 = coeffs
@@ -1326,7 +1367,68 @@ class EOS:
         )
         return P
 
+    # Outputting data to files
 
-if __name__ == "__main__":
-    eos = EOS(infile=["EvsV.dat", "EvsV2.dat"], eostype="phase_transition_pressure")
-    eos.plot_eos()
+    def to_dict(self):
+        if self.eostype == "enthalpy":
+            return {
+                "H": self.H.tolist(),
+                "au": self.au,
+                "coeffs0": self.coeffs0.tolist(),
+                "deltaH_index": self.deltaH_index,
+                "energy": self.energy.tolist(),
+                "eos_birch_fitted_cost": self.eos_birch_fitted["cost"],
+                "eos_birch_murnaghan_fitted_cost": self.eos_birch_murnaghan_fitted[
+                    "cost"
+                ],
+                "eos_murnaghan_fitted_cost": self.eos_murnaghan_fitted["cost"],
+                "eos_vinet_fitted_cost": self.eos_vinet_fitted["cost"],
+                "eostype": self.eostype,
+                "infiles": self.infiles,
+                "labels": self.labels,
+                "model": self.model,
+                "natoms": self.natoms,
+                "pressure": self.pressure.tolist(),
+                "selected_coeffs": [i.tolist() for i in self.selected_coeffs],
+                "trans_mat_reordered": self.trans_mat_reordered.tolist(),
+                "vlim": self.vlim,
+                "vlim_list": self.vlim_list,
+                "vol_array": self.vol_array.tolist(),
+                "volume": self.volume.tolist(),
+            }
+        else:
+            return {
+                "P_birch": self.P_birch.tolist(),
+                "P_birch_murnaghan": self.P_birch_murnaghan.tolist(),
+                "P_murnaghan": self.P_murnaghan.tolist(),
+                "P_vinet": self.P_vinet.tolist(),
+                "au": self.au,
+                "coeffs0": self.coeffs0,
+                "energy": self.energy.tolist(),
+                "eos_birch_fitted_cost": self.eos_birch_fitted["cost"],
+                "eos_birch_murnaghan_fitted_cost": self.eos_birch_murnaghan_fitted[
+                    "cost"
+                ],
+                "eos_murnaghan_fitted_cost": self.eos_murnaghan_fitted["cost"],
+                "eos_vinet_fitted_cost": self.eos_vinet_fitted["cost"],
+                "eostype": self.eostype,
+                "model": self.model,
+                "natoms": self.natoms,
+                "raw_data": self.raw_data,
+                "vlim": self.vlim,
+                "vol_array": self.vol_array.tolist(),
+                "volume": self.volume.tolist(),
+            }
+
+    def to_json(self, outfile="eos.json"):
+
+        wf = open(outfile, "w")
+        json.dump(self.to_dict(), wf, sort_keys=True, indent=4, separators=(",", ": "))
+        wf.close()
+
+    def to_xml(self, outfile="eos.xml"):
+        wf = open(outfile, "w")
+        xml = dicttoxml(self.to_dict())
+        dom = parseString(xml)
+        wf.write(dom.toprettyxml())
+        wf.close()
